/*------------------------------*/
/*  Intro a la programac de     */
/*  juegos, por Nacho Cabanes   */
/*                              */
/*   Nivel.cpp:                 */
/*     La pantalla del nivel 1  */
/*     Miniminer (version 0.07) */
/*     Fichero de desarrollo    */
/*                              */
/*  Comprobado con:             */
/*  - DevC++ 4.9.9.2(gcc 3.4.2) */
/*    y Allegro 4.03 - WinXP    */
/*  - gcc 4.4.3 y Allegro 4.2   */
/*    en Ubuntu 10.04           */
/*------------------------------*/

#include "Hardware.h"
#include "ElementoGraf.h"
#include "Nivel.h"

#ifdef __cplusplus
    #include <cstdlib>
    #include <cstdio>
#else
    #include <stdlib.h>
    #include <stdio.h>
#endif

#include <math.h>
#include <string>

#include <allegro.h>

Nivel::Nivel()
{
    nivelActual = 1;

    anchoImagen = 16;
    altoImagen = 16;
    margenIzq = 64;
    margenSup = 48;

    int i, j;
    
    for(i=0; i<MAXCOLS; i++)
      for (j=0; j<MAXFILAS; j++) 
      {
            fragmentoNivel[j][i] = new ElementoGraf();
            fragmentoNivel[j][i]->moverA(
                margenIzq + i*anchoImagen, margenSup + j*altoImagen);
      }
      
    leerDeFichero();

}


/** dibujarOculta: muestra el nivel sobre el fondo
 *  de pantalla, con su imagen correspondiente
 *  segun el tipo de ladrillo del que se trate
 */

void Nivel::dibujarOculta(Hardware h)
{
    int i, j;

    h.borrarOculta();

    for(i=0; i<MAXCOLS; i++)
      for (j=0; j<MAXFILAS; j++) 
        h.dibujarOculta( *fragmentoNivel[j][i] );
}



/** leerNivel: lee de fichero los datos del nivel actual
 */
void Nivel::leerDeFichero()
{
    // Si hay fichero de datos, lo leo
    FILE *fichDatos;
    char linea[MAXCOLS+2];
    char nombreFich[50];
    sprintf(nombreFich, "nivel%03d.dat", nivelActual);
    fichDatos = fopen(nombreFich, "rt"); //### Mas adelante: creciente
    if (fichDatos != NULL)
        for (int j=0; j<MAXFILAS;j++)   {
            fgets(linea,MAXCOLS+1,fichDatos);
            if (strlen(linea) < 5) // Salto los avances de linea de Windows
              fgets(linea,MAXCOLS+1,fichDatos);
            for (int i=0; i<MAXCOLS;i++)   {
              mapa[j][i] = linea[i];
              switch(linea[i]) {
                  case 'S': 
                       fragmentoNivel[j][i]->crearDesdeFichero("suelo.bmp");
                       break;
                  case 'F': 
                       fragmentoNivel[j][i]->crearDesdeFichero("sueloFragil.bmp");
                       break;
                  case 'L': 
                       fragmentoNivel[j][i]->crearDesdeFichero("ladrillo.bmp");
                       break;
                  case 'V': 
                       fragmentoNivel[j][i]->crearDesdeFichero("llave.bmp");
                       break;
                  case 'P': 
                       fragmentoNivel[j][i]->crearDesdeFichero("puerta.bmp");
                       break;
                  case 'D': 
                       fragmentoNivel[j][i]->crearDesdeFichero("deslizante.bmp");
                       break;
                  case 'A': 
                       fragmentoNivel[j][i]->crearDesdeFichero("arbol.bmp");
                       break;
                  case 'T': 
                       fragmentoNivel[j][i]->crearDesdeFichero("techo.bmp");
                       break;
                  default:
                       fragmentoNivel[j][i]->crearDesdeFichero("fondoVacio.bmp");
                       break;
              }

            }
        }

}

bool Nivel::esPosibleMover(int x1, int y1, int x2, int y2)
{
	int xCasilla1 = (x1 - margenIzq) / anchoImagen;
	int yCasilla1 = (y1 - margenSup) / altoImagen;
	int xCasilla2 = (x2 - margenIzq) / anchoImagen;
	int yCasilla2 = (y2 - margenSup) / altoImagen;
	
	if (mapa[yCasilla1][xCasilla1] != ' ')
		return false;
		
	if (mapa[yCasilla1][xCasilla2] != ' ')
		return false;

	if (mapa[yCasilla2][xCasilla2] != ' ')
		return false;

	if (mapa[yCasilla2][xCasilla1] != ' ')
		return false;
		
	return true;
}
