/*------------------------------*/
/*  Intro a la programac de     */
/*  juegos, por Nacho Cabanes   */
/*                              */
/*    miner07.cpp               */
/*                              */
/*  Ejemplo:                    */
/*   "MiniMiner" (version 0.07) */
/*                              */
/*  Comprobado con:             */
/*  - DevC++ 4.9.9.2(gcc 3.4.2) */
/*    y Allegro 4.03 - WinXP    */
/*  - gcc 4.4.3 y Allegro 4.2   */
/*    en Ubuntu Linux 10.04     */
/*------------------------------*/

#include "Hardware.h"
#include "ElementoGraf.h"
#include "Personaje.h"
#include "Enemigo.h"
#include "Presentacion.h"
#include "Nivel.h"
#include "Marcador.h"

/* -------------- Constantes globales ------------- */
#define ANCHOPANTALLA 640
#define ALTOPANTALLA 480

/* -------------- Variables globales -------------- */
Hardware hard;
Personaje *personaje;
Enemigo *enemigo;
Presentacion *presentacion;
Nivel *primerNivel;
Marcador *marcador;

bool partidaTerminada;
bool sesionTerminada;
int incrX = 4;
int incrY = 4;
int tecla;
int ySuelo = 232;


// Prototipos de las funciones que usaremos
void comprobarTeclas();
void moverElementos();
void comprobarColisiones();
void dibujarElementos();
void pausaFotograma();
void moverDerecha();
void moverIzquierda();
void lanzarPresentacion();
void moverEnemigo();
void dibujarFondo();



// --- Bucle principal del juego -----
void buclePrincipal() {
  partidaTerminada = false;
  do {
    comprobarTeclas();
    moverElementos();
    comprobarColisiones();
    dibujarElementos();
    pausaFotograma();
  } while (partidaTerminada != true);
}


// -- Comprobac de teclas para mover personaje o salir
void comprobarTeclas() {   

  if (hard.comprobarTecla(TECLA_ESC))
     partidaTerminada = true;

  if (hard.comprobarTecla(TECLA_DCHA) && hard.comprobarTecla(TECLA_ESPACIO))
    personaje->saltarDerecha();
  else if (hard.comprobarTecla(TECLA_IZQD) && hard.comprobarTecla(TECLA_ESPACIO))
    personaje->saltarIzquierda();
  else if (hard.comprobarTecla(TECLA_ESPACIO))
    personaje->saltar();
  else if (hard.comprobarTecla(TECLA_DCHA))
     personaje->moverDerecha( *primerNivel );
  else if (hard.comprobarTecla(TECLA_IZQD))
     personaje->moverIzquierda( *primerNivel );
}


// -- Mover otros elementos del juego 
void moverElementos() {
  enemigo->mover();
  personaje->mover( *primerNivel );
}


// -- Comprobar colisiones de nuestro elemento con otros, o disparos con enemigos, etc
void comprobarColisiones() {
  // Colisiones de personaje con enemigo: recolocar y perder vida
  if (personaje->colisionCon( *enemigo ))
  {
    personaje->disminuirVidas();
    marcador->indicarVidas(personaje->getVidas());
    personaje->reiniciarPosicion();
    enemigo->reiniciarPosicion();    
    hard.pausa(100);
    if (personaje->getVidas() == 0)
    {
      partidaTerminada = true;
      personaje->reiniciarVidas();  // Para la siguiente partida
    }
  }  
}


// -- Dibujar elementos en pantalla
void dibujarElementos() {

  hard.borrarOculta();
  primerNivel->dibujarOculta(hard);
  marcador->dibujarOculta(hard);
  hard.dibujarOculta( *enemigo );
  hard.dibujarOculta( *personaje );
  hard.visualizarOculta();
}


// -- Pausa hasta el siguiente fotograma
void pausaFotograma() {
  // Para 25 fps: 1000/25 = 40 milisegundos de pausa
  hard.pausa(40);
}


// -- Funciones que no son de la logica de juego, sino de 
// funcionamiento interno de otros componentes

// -- Pantalla de presentacion
void lanzarPresentacion() {
  presentacion->mostrar(hard);
}


/* -------------- Rutina de inicializacin -------- */
int inicializa()
{
    hard.inicializar(640,480);
    personaje = new Personaje();
    enemigo = new Enemigo();
    presentacion = new Presentacion();
    primerNivel = new Nivel();
    marcador = new Marcador();
    marcador->indicarVidas(personaje->getVidas());

   // Y termino indicando que no ha habido errores
   return 0;
}



/* -------------- Rutina de inicializacin -------- */
void reiniciarPartida()
{
    personaje->reiniciarPosicion();
    personaje->reiniciarVidas();
    enemigo->reiniciarPosicion();
    marcador->indicarVidas(personaje->getVidas());
}


/* ------------------------------------------------ */
/*                                                  */
/* -------------- Cuerpo del programa ------------- */

int main()
{
    // Intento inicializar
    if (inicializa() != 0)
        exit(1);
        
    sesionTerminada = false;
    do
    {
        lanzarPresentacion();
        sesionTerminada = ( presentacion->escogidoFin() );
        if (! sesionTerminada)
        {
            buclePrincipal();
            reiniciarPartida();
        }
    }
    while (! sesionTerminada);

    return 0;
}

            /* Termino con la "macro" que me pide Allegro */
END_OF_MAIN();
